package robust.pm.android;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import robust.pm.dbmanager.DBCPConn;
import robust.pm.android.ProjectTaskEntity;

public class ProjectTaskDAO {
	public boolean insertDB(ProjectTaskEntity projectTask) {
		Connection conn = null;
		PreparedStatement pstmt = null;

		String sql = "insert into ttask(prj_id, task_nm, task_st_dt, task_ed_dt, day_cnt, before_task_no, reg_dtm)" +
				" values(?,?,?,?,?,?,?)";

		try {
			conn = DBCPConn.getConnection("pool");
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, projectTask.getProject_id());
			pstmt.setString(2, projectTask.getTask_nm());
			pstmt.setString(3, projectTask.getTask_st_dt());
			pstmt.setString(4, projectTask.getTask_ed_dt());
			pstmt.setInt(5, projectTask.getDay_cnt());
			pstmt.setInt(6, projectTask.getBefore_task_no());
			pstmt.setString(7, projectTask.getReg_dtm());

			pstmt.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
			return false;
		} finally {
			try{
				pstmt.close();
				conn.close();
			}catch(SQLException sx) {}
		}

		return true;
	}

	public boolean updateDB(ProjectTaskEntity projectTask) {
		Connection conn = null;
		PreparedStatement pstmt = null;

		String sql = "update ttask set task_nm=?,task_st_dt=?,task_ed_dt=?,day_cnt=? where task_no=?";

		try {
			conn = DBCPConn.getConnection("pool");
			pstmt = conn.prepareStatement(sql);
			pstmt.setString(1, projectTask.getTask_nm());
			pstmt.setString(2, projectTask.getTask_st_dt());
			pstmt.setString(3, projectTask.getTask_ed_dt());
			pstmt.setInt(4, projectTask.getDay_cnt());
			pstmt.setInt(5, projectTask.getTask_no());
			pstmt.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
			return false;
		} finally {
			try{
				pstmt.close();
				conn.close();
			}catch(SQLException sx) {}
		}

		return true;
	}

	public boolean deleteDB(int task_no) {
		Connection conn = null;
		PreparedStatement pstmt = null;

		String sql = "delete from ttask where task_no=?";

		try {
			conn = DBCPConn.getConnection("pool");
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, task_no);
			pstmt.executeUpdate();
		} catch (SQLException e) {
			e.printStackTrace();
			return false;
		} finally {
			try{
				pstmt.close();
				conn.close();
			}catch(SQLException sx) {}
		}

		return true;
	}

	public ProjectTaskEntity get_UserDB(int prj_id, int task_no) {
		Connection conn = null;
		PreparedStatement pstmt = null;

		String sql = "select * from ttask where prj_id=? and task_no=?";
		ProjectTaskEntity projectTask = new ProjectTaskEntity();

		try {
			conn = DBCPConn.getConnection("pool");
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, prj_id);
			pstmt.setInt(2, task_no);
			ResultSet rs = pstmt.executeQuery();

			rs.next();

			projectTask.setProject_id(rs.getInt("prj_id"));
			projectTask.setTask_no(rs.getInt("task_no"));
			projectTask.setTask_nm(rs.getString("task_nm"));
			projectTask.setTask_st_dt(rs.getString("task_st_dt"));
			projectTask.setTask_ed_dt(rs.getString("task_ed_dt"));
			projectTask.setDay_cnt(rs.getInt("day_cnt"));
			projectTask.setComp_rate(rs.getInt("comp_rate"));
			projectTask.setBefore_task_no(rs.getInt("before_task_no"));
			projectTask.setReg_dtm(rs.getString("reg_dtm"));
			rs.close();
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			try{
				pstmt.close();
				conn.close();
			}catch(SQLException sx) {}
		}
		return projectTask;
	}
	
	// ֱٿ ϵ Record Task_no ˱  Լ.
	//ttask_mem ̺ task_no     ϱ  . -2010.10.03 -
	public int get_recentRcord(){
		Connection conn = null;
		PreparedStatement pstmt = null;

		// ֱٿ ϵ record ηùȣ  .
		String sql = "select * from ttask order by task_no DESC";
		int task_no=0;
		
		try {
			conn = DBCPConn.getConnection("pool");
			pstmt = conn.prepareStatement(sql);
			ResultSet rs = pstmt.executeQuery();

			rs.next();

			task_no=rs.getInt("task_no");

			rs.close();
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			try{
				pstmt.close();
				conn.close();
			}catch(SQLException sx) {}
		}

		return task_no;
	}
	
	public ArrayList<ProjectTaskEntity> get_UserDBList(int prj_id) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		
		ArrayList<ProjectTaskEntity> datas = new ArrayList<ProjectTaskEntity>();

		String sql = "select * from ttask where prj_id=?";

		try {
			conn = DBCPConn.getConnection("pool");
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, prj_id);
			ResultSet rs = pstmt.executeQuery();

			while (rs.next()) {
				ProjectTaskEntity projectTask = new ProjectTaskEntity();

				projectTask.setProject_id(rs.getInt("prj_id"));
				projectTask.setTask_no(rs.getInt("task_no"));
				projectTask.setTask_nm(rs.getString("task_nm"));
				projectTask.setTask_st_dt(rs.getString("task_st_dt"));
				projectTask.setTask_ed_dt(rs.getString("task_ed_dt"));
				projectTask.setDay_cnt(rs.getInt("day_cnt"));
				projectTask.setComp_rate(rs.getInt("comp_rate"));
				projectTask.setBefore_task_no(rs.getInt("before_task_no"));
				projectTask.setReg_dtm(rs.getString("reg_dtm"));
				datas.add(projectTask);
			}
			rs.close();

		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			try{
				pstmt.close();
				conn.close();
			}catch(SQLException sx) {}
		}

		return datas;
	}
	
	public ArrayList<ProjectTaskEntity> get_UserDBListSort(int prj_id) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		
		ArrayList<ProjectTaskEntity> datas = new ArrayList<ProjectTaskEntity>();

		String sql = "select * from ttask where prj_id=? order by task_ed_dt DESC";

		try {
			conn = DBCPConn.getConnection("pool");
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, prj_id);
			ResultSet rs = pstmt.executeQuery();

			while (rs.next()) {
				ProjectTaskEntity projectTask = new ProjectTaskEntity();

				projectTask.setProject_id(rs.getInt("prj_id"));
				projectTask.setTask_no(rs.getInt("task_no"));
				projectTask.setTask_nm(rs.getString("task_nm"));
				projectTask.setTask_st_dt(rs.getString("task_st_dt"));
				projectTask.setTask_ed_dt(rs.getString("task_ed_dt"));
				projectTask.setDay_cnt(rs.getInt("day_cnt"));
				projectTask.setComp_rate(rs.getInt("comp_rate"));
				projectTask.setBefore_task_no(rs.getInt("before_task_no"));
				projectTask.setReg_dtm(rs.getString("reg_dtm"));
				datas.add(projectTask);
			}
			rs.close();

		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			try{
				pstmt.close();
				conn.close();
			}catch(SQLException sx) {}
		}

		return datas;
	}
	
	public ArrayList<ProjectTaskEntity> get_UserDBList() {
		Connection conn = null;
		PreparedStatement pstmt = null;
		
		ArrayList<ProjectTaskEntity> datas = new ArrayList<ProjectTaskEntity>();

		String sql = "select * from ttask";

		try {
			conn = DBCPConn.getConnection("pool");
			pstmt = conn.prepareStatement(sql);
			ResultSet rs = pstmt.executeQuery();

			while (rs.next()) {
				ProjectTaskEntity projectTask = new ProjectTaskEntity();

				projectTask.setProject_id(rs.getInt("prj_id"));
				projectTask.setTask_no(rs.getInt("task_no"));
				projectTask.setTask_nm(rs.getString("task_nm"));
				projectTask.setTask_st_dt(rs.getString("task_st_dt"));
				projectTask.setTask_ed_dt(rs.getString("task_ed_dt"));
				projectTask.setDay_cnt(rs.getInt("day_cnt"));
				projectTask.setComp_rate(rs.getInt("comp_rate"));
				projectTask.setBefore_task_no(rs.getInt("before_task_no"));
				projectTask.setReg_dtm(rs.getString("reg_dtm"));
				datas.add(projectTask);
			}
			rs.close();

		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			try{
				pstmt.close();
				conn.close();
			}catch(SQLException sx) {}
		}

		return datas;
	}
}