package robust.pm.notice.service;

import java.sql.Connection;
import java.sql.SQLException;
import java.text.DecimalFormat;
import java.util.Date;

import robust.pm.notice.dao.*;
import robust.pm.notice.model.*;
import robust.pm.dbmanager.*;

public class qReplyArticleService {

	private static qReplyArticleService instance = new qReplyArticleService();
	public static qReplyArticleService getInstance() {
		return instance;
	}

	private qReplyArticleService() {
	}

	public qArticle reply(qReplyingRequest replyingRequest)
			throws qArticleNotFoundException, qCannotReplyArticleException,
			qLastChildAleadyExistsException {
		Connection conn = null;

		qArticle article = replyingRequest.toArticle();

		try {
			conn = DBCPConn.getConnection("pool");
			conn.setAutoCommit(false);

			qArticleDao articleDao = qArticleDao.getInstance();
			qArticle parent = articleDao.selectById(conn, replyingRequest
					.getParentArticleId());

			try {
				checkParent(parent, replyingRequest.getParentArticleId());
			} catch (Exception e) {
				JdbcUtil.rollback(conn);
				if (e instanceof qArticleNotFoundException) {
					throw (qArticleNotFoundException)e;
				} else if (e instanceof qCannotReplyArticleException) {
					throw (qCannotReplyArticleException)e;
				} else if (e instanceof qLastChildAleadyExistsException) {
					throw (qLastChildAleadyExistsException)e;
				}
			}

			String searchMaxSeqNum = parent.getSequenceNumber();
			String searchMinSeqNum = getSearchMinSeqNum(parent);

			String lastChildSeq = articleDao.selectLastSequenceNumber(conn,
					searchMaxSeqNum, searchMinSeqNum);

			String sequenceNumber = getSequenceNumber(parent, lastChildSeq);

			article.setGroupId(parent.getGroupId());
			article.setSequenceNumber(sequenceNumber);
			article.setPostingDate(new Date());

			int articleId = articleDao.insert(conn, article);
			if (articleId == -1) {
				throw new RuntimeException("DB  ȵ: " + articleId);
			}
			conn.commit();
			article.setId(articleId);
			return article;
		} catch (SQLException e) {
			JdbcUtil.rollback(conn);
			throw new RuntimeException("DB ۾ : " + e.getMessage(), e);
		} finally {
			if (conn != null) {
				try {
					conn.setAutoCommit(true);
				} catch (SQLException e) {
				}
			}
			JdbcUtil.close(conn);
		}
	}

	private void checkParent(qArticle parent, int parentId)
			throws qArticleNotFoundException, qCannotReplyArticleException {
		if (parent == null) {
			throw new qArticleNotFoundException(
					"θ  : " + parentId);
		}

		int parentLevel = parent.getLevel();
		if (parentLevel == 3) {
			throw new qCannotReplyArticleException(
					"  ۿ    ϴ:" + parent.getId());
		}
	}

	private String getSearchMinSeqNum(qArticle parent) {
		String parentSeqNum = parent.getSequenceNumber();
		DecimalFormat decimalFormat = 
			new DecimalFormat("0000000000000000");
		long parentSeqLongValue = Long.parseLong(parentSeqNum);
		long searchMinLongValue = 0;
		switch (parent.getLevel()) {
		case 0:
			searchMinLongValue = parentSeqLongValue / 1000000L * 1000000L;
			break;
		case 1:
			searchMinLongValue = parentSeqLongValue / 10000L * 10000L;
			break;
		case 2:
			searchMinLongValue = parentSeqLongValue / 100L * 100L;
			break;
		}
		return decimalFormat.format(searchMinLongValue);
	}

	private String getSequenceNumber(qArticle parent, String lastChildSeq)
			throws qLastChildAleadyExistsException {
		long parentSeqLong = Long.parseLong(parent.getSequenceNumber());
		int parentLevel = parent.getLevel();

		long decUnit = 0;
		if (parentLevel == 0) {
			decUnit = 10000L;
		} else if (parentLevel == 1) {
			decUnit = 100L;
		} else if (parentLevel == 2) {
			decUnit = 1L;
		}

		String sequenceNumber = null;

		DecimalFormat decimalFormat = 
			new DecimalFormat("0000000000000000");
		if (lastChildSeq == null) { // 亯  
			sequenceNumber = decimalFormat.format(parentSeqLong - decUnit);
		} else { // 亯  
			//  亯  Ȯ
			String orderOfLastChildSeq = null;
			if (parentLevel == 0) {
				orderOfLastChildSeq = lastChildSeq.substring(10, 12);
				sequenceNumber = lastChildSeq.substring(0, 12) + "9999";
			} else if (parentLevel == 1) {
				orderOfLastChildSeq = lastChildSeq.substring(12, 14);
				sequenceNumber = lastChildSeq.substring(0, 14) + "99";
			} else if (parentLevel == 2) {
				orderOfLastChildSeq = lastChildSeq.substring(14, 16);
				sequenceNumber = lastChildSeq;
			}
			if (orderOfLastChildSeq.equals("00")) {
				throw new qLastChildAleadyExistsException(
						" ڽ  ̹ մϴ:" + lastChildSeq);
			}
			long seq = Long.parseLong(sequenceNumber) - decUnit;
			sequenceNumber = decimalFormat.format(seq);
		}
		return sequenceNumber;
	}
}
