package robust.pm.servlet;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Map;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletInputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import robust.pm.property.CSProperty;
import robust.pm.servlet.multipart.DefaultFileRenamePolicy;
import robust.pm.logger.CSLogger;

/**
 * Title:
 * @version 1.0
 */

public class HttpServletRequestImpl implements HttpServletRequest {


  private HttpServletRequest req;
  private MultipartRequest multi;
  boolean isMultiPart = false;
  /**
   * attach file size limit (megabyte) : default 10M
   */
  private int ATTACH_SIZE_LIMIT = 30;
  private String ATTACH_DIR = null;
  
  /**    */
  private String	convert	= CSProperty.getInstance().getValue("LANG_SET");

  public HttpServletRequestImpl(HttpServletRequest req){
	  this(req, null, null);
  }
  
  public HttpServletRequestImpl(HttpServletRequest req, String attach_limit_s, String tmp_data_dir) {
	  this(req, attach_limit_s, tmp_data_dir, true);
  }

  /**
   * HttpServletRequestImpl
   * @param req
   * @param attach_limit_s
   * @param tmp_data_dir
   * @param bAttachFileAutoRename MultiPart Upload ϶ ȿ
   */
  public HttpServletRequestImpl(HttpServletRequest req, String sAttachLimitSize, String sTmpDataDir, boolean bAttachFileAutoRename) {
    this.req = req;
    
    if(sAttachLimitSize != null){
      try{
        ATTACH_SIZE_LIMIT = Integer.parseInt(sAttachLimitSize);
      }catch(NumberFormatException exp){
        System.err.println("[HttpServletRequestImpl] DATA_LIMIT Number Format Exception : " + sAttachLimitSize);
      }
    }
    if(sTmpDataDir != null){
      ATTACH_DIR = sTmpDataDir;
    }else{
      // if not defined, use system temporary directory
      ATTACH_DIR = System.getProperty("java.io.tmpdir");
    }

    int m_idx;
    if (req.getContentType() == null)
      m_idx = -1;
    else
      m_idx = req.getContentType().toLowerCase().indexOf("multipart/form");

    if(m_idx >= 0){
      try{
        parseMultiPart(bAttachFileAutoRename);
      }catch(Exception e){
        System.err.println("multipart data error e="+e);
        e.printStackTrace();
      }
    }
    
    //printInputParameter(req);
  }
  
  /**
   * Է Ķ͸ .
   * @param hsreq  HttpServletRequest
   */
  public void printInputParameter(HttpServletRequest hsreq) {
	  
	  StringBuffer sb = new StringBuffer();
	  Enumeration pEnum = null;
	  
	  if(hsreq == null)  CSLogger.info("HttpServletRequest is NULL!!"); 
	   
	  // 1.   
	  sb.append("*** ").append(hsreq.getRequestURI()).append("***\n");
	  
      // 2.  
	  sb.append("\n** HEADER **\n");
	  pEnum = hsreq.getHeaderNames();
	  if(pEnum == null) sb.append("[null]\n");
	  else {
		  String sHeaderName = null;
		  while(pEnum.hasMoreElements()) {
			  sHeaderName = (String) pEnum.nextElement();
			  sb.append("[")
			      .append(sHeaderName)
			      .append("] : [")
			      .append(hsreq.getHeader(sHeaderName))
			      .append("]\n");
		  }
	  }
	  
	  // 3. Ű 
	  sb.append("\n** COOKIE **\n");
	  Cookie[] cookie = hsreq.getCookies();
	  if(cookie == null) sb.append("[null]\n");
	  else {
		  for(int i = 0; i < cookie.length; i++) {
			  sb.append("[")
			      .append(cookie[i].getName())
			      .append("] : [")
			      .append(cookie[i].getValue())
			      .append("]\n");
		  }
	  }
	  
	  // 4. Է Ķ  
	  sb.append("\n** PARAMETER **\n");
	  if(!this.isMultiPart) {
		  pEnum = hsreq.getParameterNames();
		  if(pEnum == null) sb.append("[null]\n");
		  else {
			  String sParamName = null;
			  while(pEnum.hasMoreElements()) {
				  sParamName = (String) pEnum.nextElement();
				  sb.append("[")
				      .append(sParamName)
				      .append("] : [")
				      .append(hsreq.getParameter(sParamName))
				      .append("]\n");
			  }
		  }
	  } else {
		  if(this.multi == null) sb.append("[null-1]\n");
		  else {

			  Enumeration paramKey = multi.getParameterNames();
			  while(paramKey.hasMoreElements()) {
				  
				  String paramName  = (String) paramKey.nextElement(); // ڷᰡ  쿣 while  
				  String paramValue = multi.getParameter(paramName);
				  sb.append("[")
				      .append(paramName)
				      .append("] : [")
				      .append(paramValue)
				      .append("]\n");
			  }
			  
			  Enumeration formNames = multi.getFileNames();
			  
			  while(formNames.hasMoreElements()) {
				  
				  String formName=(String)formNames.nextElement(); // ڷᰡ  쿣 while  
				  String fileName=multi.getFilesystemName(formName); //  ̸ 
	    		  
                  if(fileName != null)
    				  sb.append("[")
    				      .append(formName)
    				      .append("] : [")
    				      .append(fileName)
    				      .append("]\n");
                  else
                      sb.append("[")
                      .append(formName)
                      .append("] : [")
                      .append(fileName + " is NULL")
                      .append("]\n");
		      }
		  }
	  }
	  CSLogger.debug(sb.toString()); 
  }
  

  /**
   * parseMultiPart
   * @param bAttachFileAutoRename ÷ ε  ȭϸ   ڵ ϸ 濩 
   *
   */
  void parseMultiPart(boolean bAttachFileAutoRename){

    try{
        String savePath=ATTACH_DIR; //  丮 ()
        int sizeLimit = ATTACH_SIZE_LIMIT * 1024 * 1024 ; // Attach_size_limit : mega byte units ް  Ѿ ܹ߻

	    //System.out.println("=> parseMultiPart - savePath:"+savePath);
	    //System.out.println("=> parseMultiPart - sizeLimit:"+sizeLimit);
	        
        if(bAttachFileAutoRename) 
        	multi = new MultipartRequest(req, savePath, sizeLimit, "euc-kr", new DefaultFileRenamePolicy());
        else
        	multi = new MultipartRequest(req, savePath, sizeLimit, "euc-kr");
		
	    //   Upload Ǿ multipart óѴ. 
		this.isMultiPart = true;
	
	 	Enumeration formNames = multi.getFileNames();  //  ̸ ȯ

	 	//file upload check
	    //System.err.println("=> parseMultiPart - file upload check start");
	    
	    while(formNames.hasMoreElements()) {
	    	
	 	  String formName = (String)formNames.nextElement(); // ڷᰡ  쿣 while  

		  String fileName = multi.getFilesystemName(formName); //  ̸ 
		  
		  if(fileName == null) {   //  ε  ʾ
	        System.err.println("Any file don't be uploaded");
		  } else {  //  ε Ǿ
			fileName=new String(fileName.getBytes("8859_1"), "euc-kr"); // ѱڵ -  
		  } // end if
	    }//end while
	
    }catch(Exception e){
    	e.printStackTrace(System.err);
    }
  }

  /**
   * getFile
   * @param file_parameter_name
   * @return
   */
  public File getFile(String file_parameter_name){
    File f = null;
    if(isMultiPart){
      String filename = multi.getFilesystemName(file_parameter_name);
      if(filename != null && ATTACH_DIR != null){
        f = new File(ATTACH_DIR, filename);
      }
    }else{
      System.err.println("[HttpServletRequestImpl] this request is not multi-part");
    }
    return f;
  }
  
  /*
  *  parameter_name,  file ̱
  *    outFile  ϸ, false returnѴ.
  *
  */
  public boolean getFileTo(String param_name, File outFile)
  throws IOException
  {
    boolean result = false;
    File input_f = getFile(param_name);
    if(input_f != null){
      FileInputStream fi = new FileInputStream(input_f);
      FileOutputStream fo = new FileOutputStream(outFile);
      if(fi !=null && fo !=null){
          while(true){
                  int len = fi.available();
                  if(len <=0 ){
                          break;
                  }
                  byte[] ba = new byte[len];
                  fi.read(ba);
                  fo.write(ba);
          }
          fi.close();
          fo.flush();
          fo.close();
          result = true;
      }else{
          result = false;
      }
    }else{
      result = false;
      System.err.println("There is no attach file :"+param_name);
    }
    return result;
  }
  
  /**
  **convertToKISC5601
  **/
  private String convertTo(String str){

    if ( str == null )  return null;
    if ( convert == null || "".equals(convert)) return str;
    try {
        return new String(str.getBytes("8859_1"), convert);
    } catch (Exception e) {
        return null;
    }
	//return str;
  }
  
  /**
   * getIntParameter
   * @param p0
   * @param def
   * @return
   */
  public int getIntParameter(String p0, int def) {
    String v = getParameter(p0);
    if(v == null) {
      return def;
    }
    v = v.trim();
    try{
      return Integer.parseInt(v);
    }catch(NumberFormatException e){
      return def;
    }
  }

  /**
   * getDblParameter
   * @param p0
   * @param def
   * @return
   */
  public double getDblParameter(String p0, double def) {
    String v = getParameter(p0);
    if(v == null) {
      return def;
    }
    v = v.trim();
    try{
      return new Double(v).doubleValue();
    }catch(NumberFormatException e){
      return def;
    }
  }
  
  /**
   * getFloatParameter
   * @param p0
   * @param def
   * @return
   */
  public float getFloatParameter(String p0, float def) {
    String v = getParameter(p0);
    if(v == null) {
      return def;
    }
    v = v.trim();
    try{
      return new Float(v).floatValue();
    }catch(NumberFormatException e){
      return def;
    }
  }

  /**
  *  getParameter  int ȯϿ returnѴ.
  *  Number format ƴϰų, null  -1 returnѴ.
  */
  public int getIntParameter(String p0) {
    String v = getParameter(p0);
    if(v == null) return -1;
    try{
      return Integer.parseInt(v);
    }catch(NumberFormatException e){
      return -1;
    }
  }
  
  /**
  *  getParameter  double ȯϿ returnѴ.
  *  Number format ƴϰų, null  -1 returnѴ.
  */
  public double getDblParameter(String p0) {
    String v = getParameter(p0);
    if(v == null) return -1d;
    try{
            return new Double(v).doubleValue();
    }catch(NumberFormatException e){
            return -1d;
    }
  }
  
  /**
  *  getParameter  double ȯϿ returnѴ.
  *  Number format ƴϰų, null  -1 returnѴ.
  */
  public float getFloatParameter(String p0) {
    String v = getParameter(p0);
    if(v == null) return -1f;
    try{
      return new Float(v).floatValue();
    }catch(NumberFormatException e){
      return -1f;
    }
  }

  /**
   * getShortParameter
   * @param p0
   * @param def
   * @return
   */
  public short getShortParameter(String p0, short def) {
    String v = getParameter(p0);
    if(v == null) return def;
    try{
      return Short.parseShort(v);
    }catch(NumberFormatException e){
      return def;
    }
  }

  /**
  *  getParameter  short ȯϿ returnѴ.
  *  Number format ƴϰų, null  -1 returnѴ.
  */
  public short getShortParameter(String p0) {
    String v = getParameter(p0);
    if(v == null) return -1;
    try{
      return Short.parseShort(v);
    }catch(NumberFormatException e){
      return -1;
    }
  }

  /**
   * getParameter
   * @param p0
   * @param def
   * @return
   */
  public String getParameter(String p0, String def) {
    String s = getParameter(p0);
    return s==null ? def : s;
  }

  /**
   * getParameter
   */
  public String getParameter(String p0) {
    String result = null;
    if(isMultiPart) {
      result=multi.getParameter(p0);
    }else{
      result=req.getParameter(p0);
    }
    if(result != null){
      result = convertTo(result);
    }
    return result;
  }
  /** @deprecated */
  public boolean isRequestedSessionIdFromUrl(){return req.isRequestedSessionIdFromUrl();}
  public boolean isRequestedSessionIdFromURL(){return req.isRequestedSessionIdFromURL();}
  public boolean isRequestedSessionIdFromCookie(){return req.isRequestedSessionIdFromCookie();}
  public boolean isRequestedSessionIdValid (){return req.isRequestedSessionIdValid();}
  public HttpSession getSession() {return req.getSession();}
  public HttpSession getSession (boolean create){return req.getSession(create);}
  public String getServletPath()  {return req.getServletPath();}
  public String getRequestURI() {return req.getRequestURI();}
  public String getRequestedSessionId (){return req.getRequestedSessionId();}
  public java.security.Principal getUserPrincipal() { return req.getUserPrincipal(); }
  public boolean isUserInRole(String p0) { return req.isUserInRole(p0); }
  public String getRemoteUser() {return req.getRemoteUser();}
  public String getHeader(String name){return req.getHeader(name);}
  public String getQueryString() {return req.getQueryString();}
  public String getContextPath() { return req.getContextPath(); }
  public String getPathTranslated() {return req.getPathTranslated();}
  public String getPathInfo() {return req.getPathInfo();}
  public String getMethod() {return req.getMethod();}
  public int getIntHeader(String name){return req.getIntHeader(name);}
  public Enumeration getHeaderNames(){return req.getHeaderNames();}
  public Enumeration getHeaders(String p0) { return getHeaders(p0); }
  public long getDateHeader(String name){return req.getDateHeader(name);}
  public Cookie[] getCookies() {return req.getCookies();}
  public String getAuthType() {return req.getAuthType();}
  /** @deprecated */
  public String getRealPath(String p0){return req.getRealPath(p0);}
  public RequestDispatcher getRequestDispatcher(String p0) { return req.getRequestDispatcher(p0); }
  public boolean isSecure() { return req.isSecure(); }
  public Enumeration getLocales() { return req.getLocales(); }
  public Locale getLocale() { return req.getLocale(); }
  public void removeAttribute(String p0) { req.removeAttribute(p0); }
  public void setAttribute(String s, Object o) {req.setAttribute(s, o);}
  public String getRemoteHost(){return req.getRemoteHost();}
  public String getRemoteAddr(){return req.getRemoteAddr();}
  public BufferedReader getReader() throws IOException{return req.getReader();}
  public int getServerPort(){return req.getServerPort();}
  public String getServerName(){return req.getServerName();}
  public ServletInputStream getInputStream() throws IOException{return req.getInputStream();}
  public String getScheme(){return req.getScheme();}
  public String getProtocol(){return req.getProtocol();}
  
  /**
   * ڵ ó
   * @param Ķ͸
   */
  public String[] getParameterValues(String p0) { 
	  String[] sRtnValues = req.getParameterValues(p0);
	  
	  //System.out.println("this.convert : " + this.convert);
	  
	  for(int i = 0; sRtnValues != null && i < sRtnValues.length; i++) {
		  sRtnValues[i] = convertTo(sRtnValues[i]);
		  
		  //System.out.println("setCharacterEncoding(String p0).p0:" + sRtnValues[i]);
	  }
	  
	  return sRtnValues;
  }
  public Enumeration getParameterNames() {return req.getParameterNames();}
  public Object getAttribute(String p0){return req.getAttribute(p0);}
  public String getContentType(){return req.getContentType();}
  public int getContentLength(){return req.getContentLength();}
  public String getCharacterEncoding(){return req.getCharacterEncoding();}
  public Enumeration getAttributeNames() {return req.getAttributeNames();}
  
  // servlet 2.0
  public void setCharacterEncoding(String p0) throws UnsupportedEncodingException {
	  
	  this.convert = p0;
//	  //System.out.println("setCharacterEncoding(String p0).p0 : " + p0);
//	  //System.out.println("setCharacterEncoding(String p0).this.convert : " + this.convert);
	  
	  if(p0 != null && !"".equals(p0))
		  req.setCharacterEncoding(p0);
  }
  public Map getParameterMap() { return req.getParameterMap(); }
  public StringBuffer getRequestURL() { return req.getRequestURL(); }
  public String getLocalName() { return req.getLocalName(); }
  public int getLocalPort() { return req.getLocalPort(); }
  public String getLocalAddr() { return req.getLocalAddr(); }
  public int getRemotePort() { return req.getRemotePort(); }
  
}
